package javax.naming.spi;

import com.sun.naming.internal.FactoryEnumeration;
import com.sun.naming.internal.ResourceManager;
import javax.naming.Referenceable;
import javax.naming.Reference;
import javax.naming.directory.Attributes;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NamingException;
import java.util.Hashtable;
import javax.naming.directory.DirContext;
import javax.naming.CannotProceedException;

public class DirectoryManager extends NamingManager
{
    DirectoryManager() {
    }
    
    public static DirContext getContinuationDirContext(CannotProceedException value) throws NamingException {
        Hashtable<Object, Object> environment = value.getEnvironment();
        Hashtable hashtable;
        if (environment == null) {
            hashtable = new Hashtable(7);
        }
        else {
            hashtable = (Hashtable)environment.clone();
        }
        hashtable.put("java.naming.spi.CannotProceedException", value);
        return new ContinuationDirContext(value, hashtable);
    }
    
    public static Object getObjectInstance(Object o, Name name, Context context, Hashtable<?, ?> hashtable, Attributes attributes) throws Exception {
        ObjectFactoryBuilder objectFactoryBuilder = NamingManager.getObjectFactoryBuilder();
        if (objectFactoryBuilder != null) {
            ObjectFactory objectFactory = objectFactoryBuilder.createObjectFactory(o, hashtable);
            if ((objectFactory instanceof DirObjectFactory)) {
                return ((DirObjectFactory)objectFactory).getObjectInstance(o, name, context, hashtable, attributes);
            }
            return objectFactory.getObjectInstance(o, name, context, hashtable);
        }
        else {
            Reference reference = null;
            if ((o instanceof Reference)) {
                reference = (Reference)o;
            }
            else if ((o instanceof Referenceable)) {
                reference = ((Referenceable)(Referenceable)o).getReference();
            }
            if (reference != null) {
                String factoryClassName = reference.getFactoryClassName();
                if (factoryClassName != null) {
                    ObjectFactory objectFactoryFromReference = NamingManager.getObjectFactoryFromReference(reference, factoryClassName);
                    if ((objectFactoryFromReference instanceof DirObjectFactory)) {
                        return ((DirObjectFactory)objectFactoryFromReference).getObjectInstance(reference, name, context, hashtable, attributes);
                    }
                    if (objectFactoryFromReference != null) {
                        return objectFactoryFromReference.getObjectInstance(reference, name, context, hashtable);
                    }
                    return o;
                }
                else {
                    Object processURLAddrs = NamingManager.processURLAddrs(reference, name, context, hashtable);
                    if (processURLAddrs != null) {
                        return processURLAddrs;
                    }
                }
            }
            Object objectFromFactories = createObjectFromFactories(o, name, context, hashtable, attributes);
            return (objectFromFactories != null) ? objectFromFactories : o;
        }
    }
    
    private static Object createObjectFromFactories(Object o, Name name, Context ctx, Hashtable<?, ?> env, Attributes attributes) throws Exception {
        FactoryEnumeration factories = ResourceManager.getFactories("java.naming.factory.object", env, ctx);
        if (factories == null) {
            return null;
        }
        Object o2 = null;
        while (o2 == null && factories.hasMore()) {
            ObjectFactory objectFactory = (ObjectFactory)factories.next();
            if ((objectFactory instanceof DirObjectFactory)) {
                o2 = ((DirObjectFactory)objectFactory).getObjectInstance(o, name, ctx, env, attributes);
            }
            else {
                o2 = objectFactory.getObjectInstance(o, name, ctx, env);
            }
        }
        return o2;
    }
    
    public static DirStateFactory.Result getStateToBind(Object o, Name name, Context ctx, Hashtable<?, ?> env, Attributes attributes) throws NamingException {
        FactoryEnumeration factories = ResourceManager.getFactories("java.naming.factory.state", env, ctx);
        if (factories == null) {
            return new DirStateFactory.Result(o, attributes);
        }
        Object stateToBind = null;
        while (stateToBind == null && factories.hasMore()) {
            StateFactory stateFactory = (StateFactory)factories.next();
            if ((stateFactory instanceof DirStateFactory)) {
                stateToBind = ((DirStateFactory)stateFactory).getStateToBind(o, name, ctx, env, attributes);
            }
            else {
                Object stateToBind2 = stateFactory.getStateToBind(o, name, ctx, env);
                if (stateToBind2 == null) {
                    continue;
                }
                stateToBind = new DirStateFactory.Result(stateToBind2, attributes);
            }
        }
        return (stateToBind != null) ? stateToBind : new DirStateFactory.Result(o, attributes);
    }
}
